VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cIPCWorker"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit    'a generic (StateMachine-based) Class for IPC (Inter-Process-Communication) via SharedMemory
'this Class implements the Worker-Part (note, that both cIPC-classes can be used independently)
'Author: Olaf Schmidt (2014)

Private Declare Function IsWindow Lib "user32" (ByVal hWnd As Long) As Long
Private Declare Function timeBeginPeriod Lib "winmm" (ByVal uPeriod As Long) As Long
Private Declare Function timeEndPeriod Lib "winmm" (ByVal uPeriod As Long) As Long
Private Declare Sub RtlMoveMemory Lib "kernel32" (Dst As Any, Src As Any, ByVal bc&)
Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
Private Declare Function QueryPerformanceFrequency& Lib "kernel32" (X@)
Private Declare Function QueryPerformanceCounter& Lib "kernel32" (X@)
Private Declare Function CreateFileMappingW& Lib "kernel32" (ByVal hMapFile&, Attr As Any, ByVal Protect&, ByVal SizeHigh&, ByVal SizeLow&, ByVal MapName&)
Private Declare Function MapViewOfFile& Lib "kernel32" (ByVal hMapFile&, ByVal DesAccess&, ByVal OffsHigh&, ByVal OffsLow&, ByVal NumberOfBytesToMap&)
Private Declare Function UnmapViewOfFile& Lib "kernel32" (ByVal lpBaseAddress As Long)
Private Declare Function CloseHandle& Lib "kernel32" (ByVal hObject&)
Private Const PAGE_READWRITE& = &H4, FILE_MAP_ALL_ACCESS& = &HF001F

'**** don't change the following two defs (similar ones are used in cIPCMaster)
Public Enum eJobStateWorker
    wjobUndefined = 0     'structure needs initialization
    wjobMasterInitialized    'set from the Master-Instance
    wjobPrepared          'set from the Master-Instance
    wjobCancelled         'set from the Master-Instance
    wjobProcessing        'set from the Worker-Instance
    wjobFinished          'set from the Worker-Instance
End Enum

Private Type tShared    'a convenience-UDT, sitting at the top of the shared mem-area
    UserDataLen As Long
    MasterHwnd As Long
    JobProgress As Single
    JobPreparedHPSec As Double
    JobStartedHPSec As Double
    JobFinishedHPSec As Double
    JobState As eJobStateWorker
End Type
'**** end of shared Based-defs

Private hMapFile&, BaseAddr&, T As tShared
Attribute BaseAddr.VB_VarUserMemId = 1073938432
Attribute T.VB_VarUserMemId = 1073938432

Public Function InitWithCommandLineJobKey(JobKey As String) As Boolean
    Dim UsrDatLen As Long
    UsrDatLen = Split(JobKey, "_")(0)
    hMapFile = CreateFileMappingW(-1, ByVal 0&, PAGE_READWRITE, 0&, LenB(T) + UsrDatLen, StrPtr(JobKey))
    If hMapFile Then BaseAddr = MapViewOfFile(hMapFile, FILE_MAP_ALL_ACCESS&, 0&, 0&, 0&)
    If BaseAddr Then InitWithCommandLineJobKey = True: timeBeginPeriod 1
End Function

Public Function WaitForNextPreparedJob() As Boolean
    Do Until JobState = wjobPrepared    'wait for the Master-initiated next state-switch
        Wait 1: If MasterWantsToClose Then Exit Function
    Loop
    JobState = wjobProcessing    'switch to the next state
    WaitForNextPreparedJob = True
End Function

Public Property Get JobStateFriendlyName() As String
    If BaseAddr Then JobStateFriendlyName = Choose(JobState + 1, "Undefined", "MasterInitialized", "Prepared", "Cancelled", "Processing", "Finished")
End Property
Public Property Get JobState() As eJobStateWorker
    If BaseAddr Then JobState = ReadInternalType.JobState
End Property
Public Property Let JobState(ByVal RHS As eJobStateWorker)
    If BaseAddr = 0 Then Exit Property
    T = ReadInternalType
    T.JobState = RHS
    Select Case T.JobState
        Case wjobProcessing: T.JobStartedHPSec = HPTimer
        Case wjobFinished: T.JobFinishedHPSec = HPTimer
    End Select
    WriteInternalType T
End Property

Public Property Get JobProgress() As Single
    If BaseAddr Then JobProgress = ReadInternalType.JobProgress
End Property
Public Property Let JobProgress(ByVal RHS As Single)
    If BaseAddr Then T = ReadInternalType: T.JobProgress = RHS: WriteInternalType T
End Property

Public Property Get JobPreparedHPSec() As Double
    If BaseAddr Then JobPreparedHPSec = ReadInternalType.JobPreparedHPSec
End Property
Public Property Get JobStartedHPSec() As Double
    If BaseAddr Then JobStartedHPSec = ReadInternalType.JobStartedHPSec
End Property
Public Property Get JobFinishedHPSec() As Double
    If BaseAddr Then JobFinishedHPSec = ReadInternalType.JobFinishedHPSec
End Property

Public Property Get MasterHwnd() As Long
    If BaseAddr Then MasterHwnd = ReadInternalType.MasterHwnd
End Property

Public Property Get MasterWantsToClose() As Boolean
    MasterWantsToClose = (IsWindow(MasterHwnd) = 0)
End Property

Public Property Get UserDataLen() As Long
    If BaseAddr Then UserDataLen = ReadInternalType.UserDataLen
End Property
Public Property Get UserDataPtr() As Long
    If BaseAddr Then UserDataPtr = BaseAddr + LenB(T)
End Property
Public Sub UserDataRead(ByVal pDst As Long, ByVal ByteLen As Long, Optional ByVal Offs As Long)
    If BaseAddr Then RtlMoveMemory ByVal pDst, ByVal UserDataPtr + Offs, ByteLen
End Sub
Public Sub UserDataWrite(ByVal pSrc As Long, ByVal ByteLen As Long, Optional ByVal Offs As Long)
    If BaseAddr Then RtlMoveMemory ByVal UserDataPtr + Offs, ByVal pSrc, ByteLen
End Sub

Public Sub Wait(ByVal msec As Long)
    Sleep msec: DoEvents
End Sub

Public Function HPTimer#()
    Dim X@, Frq@
    QueryPerformanceFrequency Frq
    If QueryPerformanceCounter(X) Then HPTimer = X / Frq
End Function

Private Function ReadInternalType() As tShared
    If BaseAddr Then RtlMoveMemory ReadInternalType, ByVal BaseAddr, LenB(ReadInternalType)
End Function
Private Sub WriteInternalType(T As tShared)
    If BaseAddr Then RtlMoveMemory ByVal BaseAddr, T, LenB(T)
End Sub

Private Sub Class_Terminate()
    timeEndPeriod 1
    If BaseAddr Then UnmapViewOfFile BaseAddr: BaseAddr = 0
    If hMapFile Then CloseHandle hMapFile: hMapFile = 0
End Sub

